<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | PHP version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available through the world-wide-web at                              |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Paul M. Jones <pmjones@ciaweb.net>                          |
// +----------------------------------------------------------------------+
//
// $Id: Text_Wiki.php,v 1.18 2003/12/04 20:38:21 pmjones Exp $

require_once 'Text_Wiki/Rule.php';


/**
* 
* This is the "master" class for handling the management and convenience
* functions to transform Wiki-formatted text.
* 
* @author Paul M. Jones <pmjones@ciaweb.net>
* 
* @version 0.5 alpha
* 
*/

class Text_Wiki {
    
    
    /**
    * 
    * The array of rules to apply to the source text, in order.  Each of
    * these must exist as a "Text_Wiki_Rule_*.php" class in the default
    * rule directory or the user-defined rule directory.
    * 
    * @access public
    * 
    * @var array
    * 
    */
    
    var $rules = array(
    
        // pre-filters
        'prefilter',
        'delimiter',
        
        // non-paragraph block elements
        'codeblock',
        'htmlblock',
        'heading',
        'horiz',
        'blockquote',
        'deflist',
        'table',
        'list',
        
        // paragraph block elements
        'paragraph',
        
        // inline elements
        'raw',
        'url',
        'interwiki',
        'wikifreelink',
        'wiki',
        'strong',
        'emphasis',
        'tt',
        
        // plugins
        'toc',
        
        // post-filters
        'entities',
        'tighten'
    ); 
    
    /**
    * 
    * An associative array of Interwiki mappings where the key is the
    * Interwiki name as entered on a wiki page, and the value is the
    * replacement URL for the target wiki web.
    * 
    * @access public
    * 
    * @var array
    * 
    */
    
    var $interwiki = array(
        'MeatBall' => 'http://www.usemod.com/cgi-bin/mb.pl?',
        'Advogato' => 'http://advogato.org/',
        'Wiki' => 'http://c2.com/cgi/wiki?'
    );
    
    
    /**
    * 
    * An array of all pages that currently exist in the wiki.  The source
    * of the pages does not matter (database, file system, whatever).  All
    * that Text_Wiki needs to know is what pages are already in the system
    * so that it can decide what kind of link to show when that page name
    * appears in the source text.
    * 
    * @access public
    * 
    * @var array
    * 
    */
    
    var $pages = array(); 
    
    
    /**
    * 
    * The URL used to view an existing page in the wiki; the page name
    * will be appended to this base URL.
    * 
    * @access public
    * 
    * @var string
    * 
    */
    
    var $view_url = 'http://example.com/index.php?page=';
    
    
    /**
    * 
    * The URL used to create a page that does not exist in the wiki; the
    * page name will be appended to this base URL.
    * 
    * @access public
    * 
    * @var string
    * 
    */
    
    var $new_url = 'http://example.com/new.php?page=';
    
    
    /**
    * 
    * When a page does not exist in the wiki, this is the text for the
    * link to the "create" URL. (Typically, the page name itself is not
    * the linked text; instead, a question mark or an inline image is
    * used to indicate the page does not exist.)
    * 
    * @access public
    * 
    * @var string
    * 
    */
    
    var $new_text = '?';
    
    
    /**
    * 
    * The directory where user-defined rule class files reside. 
    * Defaults to the current directory.  Text_Wiki first looks for
    * rules in the user-defined rule path, then looks in the default
    * path; this allows the developer to override existing rule class
    * files.
    * 
    * @access public
    * 
    * @var string
    * 
    * @see $this->_default_path
    * 
    */
    
    var $rule_path = '';
    
    
    /**
    * 
    * The delimiter that surrounds a token number embedded in the source
    * wiki text.
    * 
    * @access public
    * 
    * @var string
    * 
    */
    
    var $delim = "\xFF"; 
    
    
    /**
    * 
    * The default directory for rule class files.  The constructor will
    * build this automatically based on where Text_Wiki.php is in the
    * file system.  The default is 'Text_Wiki/rules/'.  Text_Wiki first
    * looks for rules in the user-defined rule path, then looks in the
    * default path; this allows the developer to override existing
    * rule class files.
    * 
    * @access private
    * 
    * @var string
    * 
    * @see $this->rule_path
    * 
    */
    
    var $_default_path = '';
    
    
    /**
    * 
    * An array of tokens generated by rules as the source text is
    * parsed.
    * 
    * As Text_Wiki applies rule classes to the source text, it will
    * replace portions of the text with a delimited token number.  This
    * is the array of those tokens, representing the replaced text and
    * any options set by the parser for that replaced text.
    * 
    * The tokens array is seqential; each element is itself a sequential
    * array where element 0 is the name of the rule that generated the
    * token, and element 1 is an associative array where the key is an
    * option name and the value is an option value.
    * 
    * @access private
    * 
    * @var string
    * 
    */
    
    var $_tokens = array();
    
    
    /**
    * 
    * The source text to which rules will be applied.  This text will be
    * transformed in-place, which means that it will change as the rules
    * are applied.
    * 
    * @access private
    * 
    * @var string
    * 
    */
    
    var $_source = '';
    
    
    /**
    * 
    * Text_Wiki creates one instance of every rule that is applied to
    * the source text; this array holds those instances.  The array key
    * is the rule name, and the array value is an instance of the rule
    * class.
    * 
    * @access private
    * 
    * @var string
    * 
    */
    
    var $_rule_obj = array();
    
    
    /**
    * 
    * Constructor.
    * 
    * @access public
    * 
    * @param array $options An associative array of options where the
    * key is the option name and the value is the option value.  Each
    * option key corresponds to a public property of Text_Wiki; e.g.,
    * 'rules', 'interwiki', 'pages', 'rule_path', and so on.
    *     
    */
    
    function Text_Wiki($options = array())
    {
        // set the default path for 
        $this->_default_path = dirname(__FILE__) . '/Text_Wiki/rules/';
        
        if (isset($options) && is_array($options)) {
            foreach ($options as $key => $val) {
                if (substr($key, 0, 1) != '_') {
                    $this->$key = $val;
                }
            }
        }
    }
    
    
    /**
    * 
    * Transforms the source text in-place.
    * 
    * First, the method parses the source text, applying rules to the
    * text as it goes.  These rules will modify the source text
    * in-place, replacing some text with delimited tokens (and
    * populating the $this->_tokens array as it goes).
    * 
    * Next, the method renders the in-place tokens into the requested
    * output format.
    * 
    * Finally, the method returns the transformed text.  Note that the
    * source text is transformed in place; once it is transformed, it is
    * no longer the same as the original source text.
    * 
    * @access public
    * 
    * @param string $text The source text to which wiki rules should be
    * applied, both for parsing and for rendering.
    * 
    * @param string $format The target output format, typically 'xhtml'.
    *  If a rule does not support a given format, the output from that
    * rule is rule-specific.
    * 
    * @return string The transformed wiki text.
    * 
    */
    
    function transform($text, $format = 'xhtml')
    {
        // set the object property for the source text
        $this->_source = $text;
        
        // apply the parse() method of each requested rule to the source
        // text.  load each rule class file as we go.
        foreach ($this->rules as $rule) {
            $this->_loadRule($rule);
            $this->_rule_obj[$rule]->parse();
        }
        
        // render each token from the parsed source text
        foreach ($this->_tokens as $key => $val) {
            
            // create variable names that are easy to read ;-)
            $rule = $val[0];
            $options = $val[1];
            
            // look for this delimited token number in the parsed source
            // text
            $find = $this->delim . $key . $this->delim;
            
            // get the replacement text for the token based on the
            // render*() method for the rule that created the token
            $method = "render$format";
            $replace = $this->_rule_obj[$rule]->$method($options);
            
            // replace the delimited token in the source text
            $this->_source = str_replace($find, $replace, $this->_source);
        }
        
        // return the newly transformed source text
        return $this->_source;
    }
    
    
    /**
    * 
    * Checks to see if a file exists and is readable.
    * 
    * @access private
    * 
    * @param string $file The path to a file.
    * 
    * @return bool True is the file exists and is readable, false if
    * not.
    * 
    */
    
    function _fileExists($file)
    {
        if (file_exists($file) &&
            is_readable($file)) {
            return true;
        } else {
            return false;
        }
    }
    
    
    /**
    * 
    * Loads a rule class file and creates an instance of the rule object.  Looks
    * first for rules in the user-defined rule path, then looks in the default path.
    * 
    * @access private
    * 
    * @param string $rule The rule name to load (not including the 'Text_Wiki_Rule_'
    * prefix, and not including the '.php' suffix).
    * 
    * @return bool True is the file exists and is readable, false if
    * not.
    * 
    */
    
    function _loadRule($rule)
    {
        // set up the class and file names
        $class = 'Text_Wiki_Rule_' . $rule;
        $file = $rule . '.php';
        
        // has the class been loaded?
        if (! class_exists($class)) {
            
            // no, try to load it
            if ($this->_fileExists($this->rule_path . $file)) {
            
                // rule exists in the user-defined path
                include_once $this->rule_path . $file;
                
            } elseif ($this->_fileExists($this->_default_path . $file)) {
            
                // rule exists in the default path
                include_once $this->_default_path . $file;
                
            } else {
            
                // not found in either path
                echo "rule class file not found: $file <br />";
                return false;
                
            }
        }
        
        // create an object instance of the rule
        if (! isset($this->_rule_obj[$rule])) {
            $this->_rule_obj[$rule] =& new $class($this);
        }
        
        return true;
    }
}

?>