<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | PHP version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available through the world-wide-web at                              |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Paul M. Jones <pmjones@ciaweb.net>                          |
// +----------------------------------------------------------------------+
//
// $Id: toc.php,v 1.1 2003/12/04 20:38:21 pmjones Exp $


/**
* 
* This class implements a Text_Wiki_Rule to find all heading tokens and
* build a table of contents.  The %%TOC%% tag gets replaced with a list
* of all the headings.
*
* @author Paul M. Jones <pmjones@ciaweb.net>
*
* @package Text_Wiki
*
*/

class Text_Wiki_Rule_toc extends Text_Wiki_Rule {
    
    
    /**
    * 
    * The regular expression used to parse the source text and find
    * matches conforming to this rule.  Used by the parse() method.
    * 
    * @access public
    * 
    * @var string
    * 
    * @see parse()
    * 
    */
    
    var $regex = "/%%TOC%%/m";
    
    
    /**
    * 
    * Custom parsing (have to process heading entries first).
    * 
    * @access public
    * 
    * @see Text_Wiki::parse()
    * 
    */
    
    function parse()
    {
        $this->_processHeadings();
        parent::parse();
    }
    
    
    /**
    * 
    * Generates a replacement for the matched text.  Token options are:
    * 
    * 'type' => ['collect'|'entry'] Whether this is the collection-point
    * for all entries, or a specific entry.
    *
    * 'key' => The element number for the heading-start token.
    *
    * 'level' => The heading level (1-6).
    *
    * 'text' => The text of the heading to be used as the TOC text.
    * 
    * @access public
    *
    * @param array &$matches The array of matches from parse().
    *
    * @return string A token indicating the TOC collection point.
    *
    */
    
    function process(&$matches)
    {
        return $this->addToken(array('type' => 'collect'));
    }
    
    
    /**
    * 
    * Finds all headings in the text and creates table-of-contents
    * entry tokens.  Token keys are:
    * 
    * 'type' => ['collect'|'entry'] Whether this is the collection-point
    * for all entries, or a specific entry.
    *
    * 'key' => The element number for the heading-start token.
    *
    * 'level' => The heading level (1-6).
    *
    * 'text' => The text of the heading to be used as the TOC text.
    * 
    * @access public
    *
    * @return void
    * 
    */
    
    function _processHeadings()
    {
        // loop through all tokens and get headings
        foreach ($this->_wiki->_tokens as $key => $val) {
        
            if ($val[0] == 'heading' &&
                $val[1]['type'] == 'start') {
                
                // create a TOC entry
                $options = array(
                    'type' => 'entry',
                    'key' => $key,
                    'level' => $val[1]['level'],
                    'text' => $val[1]['text']
                );
                
                // add to the tokens array
                $token = $this->addToken($options);
                
                // put the TOC entry token in front of the heading-start token
                $find = $this->_wiki->delim . $key . $this->_wiki->delim;
                $replace = $token . $find;
                
                $this->_wiki->_source = str_replace($find, $replace, $this->_wiki->_source);
                
            }
        }
    }
    
    /**
    * 
    * Renders a token into text matching the requested format.
    * 
    * @access public
    * 
    * @param array $options The "options" portion of the token (second
    * element).
    * 
    * @return string The text rendered from the token options.
    * 
    */
    
    function renderXhtml($options)
    {
        // the prefix used for anchor names
        $prefix = 'toc';
        
        // if this is a toc entry...
        if ($options['type'] == 'entry') {
            // ... generate an anchor.
            return "<a name=\"{$prefix}{$options['key']}\"></a>";
        }
        
        // if this is a collection...
        if ($options['type'] == 'collect') {
        
            // ... print out every toc entry.
        
            // the toc output.
            $output = '';
            
            // loop through each token
            foreach ($this->_wiki->_tokens as $key => $val) {
                
                // get every toc entry for heading-2 and higher
                if ($val[0] == 'toc' &&
                    $val[1]['type'] == 'entry' &&
                    $val[1]['level'] > 1) {
                    
                    // build some indenting spaces for the text
                    $indent = ($val[1]['level'] - 1) * 4;
                    $pad = str_pad('', $indent);
                    $pad = str_replace(' ', '&nbsp;', $pad);
                    
                    // add an extra linebreak in front of heading-2
                    // entries (makes for nice section separations)
                    if ($val[1]['level'] <= 2 && $output != '') {
                        $output .= "<br />\n";
                    }
                    
                    // create the entry line as a link to the toc anchor
                    $output .= "$pad<a href=\"#toc{$val[1]['key']}\">{$val[1]['text']}</a><br />\n";
                }
            }
            
            // return the table of contents
            return "<h2>Table of Contents</h2>\n<p>$output</p>";
        }
    }
}
?>